<?php
/*
 * Copyright (c) 2013, webvariants GbR, http://www.webvariants.de
 *
 * This file is released under the terms of the MIT license. You can find the
 * complete text in the attached LICENSE file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

namespace sly\ImportExport\Archive;

abstract class Base {
	const TYPE_ZIP = 1;
	const TYPE_SQL = 2;

	protected $filename;

	public function __construct($filename) {
		$this->filename = $filename;
	}

	public function getMetadata() {
		$comment = $this->readComment();

		if (!is_string($comment)) {
			return false;
		}

		return json_decode($comment, true);
	}

	public function setMetadata($data) {
		return $this->writeComment(json_encode($data));
	}

	public function getFilename() {
		return $this->filename;
	}

	abstract public function addFile($filename);
	abstract public function open();
	abstract public function close();

	abstract protected function readComment();
	abstract protected function writeComment($comment);
}
