jQuery(function($) {
	'use strict';

	var
		cntSelector    = '.form-repeat-container',
		rowSelector    = '.form-repeat-row',
		delSelector    = '.form-repeat-delete',
		addSelector    = '.form-repeat-append',
		topSelector    = '.form-repeat-top',
		upSelector     = '.form-repeat-up',
		downSelector   = '.form-repeat-down',
		bottomSelector = '.form-repeat-bottom',
		newgroup       = 1000;

	var renumberRows = function(container) {
		var
			rows     = $(rowSelector, container),
			i        = 0,
			len      = rows.length,
			isSlice  = container.data('isslice'),
			row, j;

		for (; i < len; ++i) {
			row = $(rows[i]);

			row.toggleClass('even', i%2===0)
				.toggleClass('odd', i%2===1)
				.toggleClass('first', i === 0);

			if (isSlice) {
				$('[data-nametemplate]', row).each(function() {
					var self = $(this);
					self.attr('name', self.data('nametemplate').replace('%n%', i));
				});
			}
		}

		container.toggleClass('empty', len === 0).toggleClass('nonempty', len > 1).toggleClass('single', len === 1);
	};

	// create add/delete links

	var addRowButtons = function(row) {
		var $container = $('<div class="form-repeat-controls"></div>');

		$container.append('<a href="#" class="form-repeat-top form-repeat-button"></a>');
		$container.append('<a href="#" class="form-repeat-up form-repeat-button"></a>');
		$container.append('<a href="#" class="form-repeat-down form-repeat-button"></a>');
		$container.append('<a href="#" class="form-repeat-bottom form-repeat-button"></a>');
		$container.append('<a href="#" class="form-repeat-delete form-repeat-button"></a>');
		$container.append('<a href="#" class="form-repeat-append form-repeat-button"></a>');

		row.append($container);
	}

	var containers = $(cntSelector);

	for (var i = 0, len = containers.length; i < len; ++i) {
		var
			cont = $(containers[i]),
			rows = $(rowSelector, cont),
			rlen = rows.length;

		cont.addClass(rlen === 0 ? 'empty' : 'nonempty');

		addRowButtons(cont);

		for (var j = 0; j < rlen; ++j) {
			var row = $(rows[j]);
			addRowButtons(row);
		}

		renumberRows(cont);
	}

	// remove an existing item

	$(cntSelector).delegate(delSelector, 'click', function() {
		var self = $(this), row = self.closest(rowSelector), cont = row.closest(cntSelector);

			row.remove();
			renumberRows(cont);

		return false;
	});

	// add a new row

	$(cntSelector).delegate(addSelector, 'click', function() {
		var self = $(this), row = self.closest(rowSelector), cont = self.closest(cntSelector);

		if (cont.data('isslice')) {
			var newrow = row.clone();
			row.after(newrow);
			renumberRows(cont);
		}
		else {
			$.ajax({
				'url': 'index.php?page=devutilsrepeat',
				'data': {'type':  cont.data('metatype'), 'name': cont.data('metaname'), 'group': newgroup++, 'ns': cont.data('ns') },
				'dataType': 'json',
				'success': function(data) {
					if (data.markup === undefined) return false;

					var newrow = $(data.markup);
					if (row.length === 1) {
						row.after(newrow);
					}
					else {
						cont.append(newrow);
					}
					addRowButtons(newrow);
					renumberRows(cont);
					$(newrow).trigger('rowAdded');

					sly.initWidgets(newrow);
				}
			});
		}

		return false;
	});

	// move row to the bottom

	$(cntSelector).delegate(bottomSelector, 'click', function() {
		var row = $(this).closest(rowSelector), cont = row.closest(cntSelector);

		cont.append(row);
		renumberRows(cont);

		return false;
	});

	// move row to the top

	$(cntSelector).delegate(topSelector, 'click', function() {
		var row = $(this).closest(rowSelector), cont = row.closest(cntSelector);

		cont.prepend(row);
		renumberRows(cont);

		return false;
	});

	// move row down

	$(cntSelector).delegate(downSelector, 'click', function() {
		var row = $(this).closest(rowSelector),	cont = row.closest(cntSelector);

		row.next().after(row);
		renumberRows(cont);

		return false;
	});

	// move row up

	$(cntSelector).delegate(upSelector, 'click', function() {
		var row = $(this).closest(rowSelector), cont = row.closest(cntSelector);

		row.prev().before(row);
		renumberRows(cont);

		return false;
	});
});