<?php
/*
 * Copyright (c) 2013, webvariants GbR, http://www.webvariants.de
 *
 * This file is released under the terms of the MIT license. You can find the
 * complete text in the attached LICENSE file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

/**
 * Datentyp "Benutzer"
 *
 * Dieser Datentyp kann bis zu 65k Zeichen lange einzeilige Benutzernamen als Werte
 * annehmen.
 */
class WV_Datatype_User extends WV_Datatype_Base_String implements WV_IDatatype {
	private static $autocomplete = null;

	public function renderForm(sly_Form $form, WV_IProperty $property, $value) {
		$value  = $this->decodeFormValue($property, $value);
		$params = $property->getParams();

		extract($this->deserializeParams($params));

		// Style vorberechnen

		$style = '';

		if (!empty($width)) $style[] = 'width:'.intval($width).'px';
		if (!empty($align)) $style[] = 'text-align:'.$align;
		if (!empty($css))   $style[] = $css;

		// Element einrichten

		$input = new WV_Form_User('property_'.$property->getID().'_value', $property->getTitle(), $value);

		$input->addClass('property_errorable');
		$input->setMultilingual($property->isMultilingual());
		$input->setAttribute('style', implode(';', $style));

		// Element in Slice einhängen

		if (!empty($annotation)) {
			$input->setAnnotation($annotation);
		}

		WV_Datatype::setHelpTextOnElement($form, $input, $property);
		$form->add($input);
		return $form;
	}

	/**
	 * Autocomplete aktivieren
	 *
	 * Diese Methode aktiviert für den nächsten renderConfigForm-Aufruf die Option
	 * für das Autocomplete. Um die Funktionalität zu nutzen, muss der Name des
	 * später in renderForm() zu verwendenden GET-Parameters angegeben werden,
	 * damit ein AddOn anhand dieses Parameters die Suchanfrage entgegennehmen
	 * und auswerten kann.
	 *
	 * Nach dem Aufruf von renderConfigForm() wird die Einstellung wieder
	 * zurückgesetzt.
	 */
	public static function enableAutocomplete($queryParam) {
		self::$autocomplete = $queryParam;
	}

	public function isValueContained($valueToSearch, $valueFromDatabase, array $params, $filterParams) {
		$valueFromDatabase = mb_strtolower($valueFromDatabase);
		$valueToSearch     = mb_strtolower($valueToSearch);

		return $valueToSearch === $valueFromDatabase;
	}
}
